==> cloud-4.2.0.tar.gz: cloud/modules/cloud_dashboard/cloud_dashboard/src/pages/EntityViewPage.tsx <==
import ENTITY_INFO_LIST from 'constant/entity_info_template/entity_info_template';
import { MENU_TEMPLATE_LIST } from 'constant/menu_template';
import { ROUTE_URL } from 'constant/other';
import CloudServiceProvider from 'model/CloudServiceProvider';
import MenuTemplate from 'model/MenuTemplate';
import EntityTabs from 'molecules/EntityTabs';
import FieldSelect from 'molecules/FieldSelect';
import ItemCountLabel from 'molecules/ItemCountLabel';
import PageSelector from 'molecules/PageSelector';
import EntityTable from 'organisms/EntityTable';
import SideBar from 'organisms/SideBar';
import { useEffect, useState } from 'react';
import { Button, Col, Container, Form, Row } from 'react-bootstrap';
import { useTranslation } from 'react-i18next';
import { Link, useParams } from 'react-router-dom';
import { calcPanelName, getEntityListViewUrl, getEntityTypeId } from 'service/utility';

/**
 * The button of creating cloud entity.
 *
 * @param menuTemplate The template of form menu.
 */
const CreateButtonGroup = ({ menuTemplate, cloudContext }: {
  menuTemplate: MenuTemplate,
  cloudContext: string
}) => {

  const { t } = useTranslation();

  switch (menuTemplate.cloudServiceProvider) {
    case 'k8s':
      switch (menuTemplate.entityName) {
        case 'node':
          return <></>;
        case 'event':
          return <></>;
        case 'schedule':
          return <></>;
        default:
          break;
      }
      break;
    case 'vmware':
      switch (menuTemplate.entityName) {
        case 'host':
          return <></>;
        default:
          break;
      }
      break;
    default:
      break;
  }

  const refresh = () => {
    const url = `/clouds/${menuTemplate.cloudServiceProvider}/${cloudContext}/${menuTemplate.entityName}/update`;

    fetch(url, { method: 'POST' }).then((response) => {
      if (!response.ok) {
        return;
      }
      window.location.href = `${ROUTE_URL}/${menuTemplate.cloudServiceProvider}/${cloudContext}/${menuTemplate.entityName}`;
    });
  }

  return <Form.Group>
    <Link className="btn btn-primary" to={`/${menuTemplate.cloudServiceProvider}/${cloudContext}/${menuTemplate.entityName}/create`}>
      Add {calcPanelName('', menuTemplate.cloudServiceProvider)} {menuTemplate.entityName}
    </Link>
    <Button onClick={refresh}>
      {t('Refresh')}
    </Button>
  </Form.Group>;

}

const calcDetailInfoColumn = (menuTemplate: MenuTemplate) => {
  // If the entity_info_template is not registered, it will not be detected.
  const registeredTemplateList = ENTITY_INFO_LIST.filter((e) => {
    return e.cloudServiceProvider === menuTemplate.cloudServiceProvider
      && e.entityName === menuTemplate.entityName;
  });
  if (registeredTemplateList.length < 1) {
    return undefined;
  }

  // Detect a property whose name is 'name' and sets a link to it.
  const nameColumnList = menuTemplate.entityColumn.filter((e) => {
    return e.name === 'name';
  })
  if (nameColumnList.length > 0) {
    return nameColumnList[0].name;
  }

  // Detect a property whose name contains 'name' and set links to it.
  const likeNameColumnList = menuTemplate.entityColumn.filter((e) => {
    return e.name.includes('name');
  })
  if (likeNameColumnList.length > 0) {
    return likeNameColumnList[0].name;
  }

  // Default value when automatic detection is not possible.
  return undefined;
}

const EntityViewPage = () => {
  const params: {
    cloudServiceProvider: string,
    cloudContext?: string,
    entityName: string
  } = useParams();

  const [namespace, setNamespace] = useState<string>('');
  const [namespaceName, setNamespaceName] = useState<string>('');
  const [itemCount, setItemCount] = useState(0);
  const [pageIndex, setPageIndex] = useState(0);

  useEffect(() => {
    if (namespace !== '') {
      setNamespaceName('');
    }
  }, [namespace]);

  useEffect(() => {
    if (namespaceName !== '') {
      setNamespace('');
    }
  }, [namespaceName]);

  useEffect(() => {
    window.scrollBy(0, -window.pageYOffset);
  }, [pageIndex]);

  const menuTemplate = MENU_TEMPLATE_LIST.filter((m) => {
    return m.cloudServiceProvider === params.cloudServiceProvider
      && m.entityName === params.entityName;
  })[0];
  const namespaceFlg = menuTemplate.entityColumn.map((c) => c.labelName).includes('Namespace');
  const namespaceNameFlg = menuTemplate.entityColumn.map((c) => c.labelName).includes('Namespace Name');
  const entityTypeId = getEntityTypeId(menuTemplate);
  const detailInfoColumn = calcDetailInfoColumn(menuTemplate);

  return <>
    <SideBar />
    <EntityTabs menuType={menuTemplate.cloudServiceProvider} entityName={menuTemplate.entityName} cloudContext={params.cloudContext} />
    <Container fluid className="px-0">
      <Row className="mx-0">
        <Col>
          <Form>
            {
              params.cloudContext !== undefined
                ? <CreateButtonGroup menuTemplate={menuTemplate} cloudContext={params.cloudContext} />
                : <></>
            }
            {
              namespaceFlg
                ? <Form.Group style={{ marginTop: '2rem' }}>
                  <Form.Label>Namespace</Form.Label>
                  <FieldSelect
                    columnKey="namespace"
                    columnName={namespace}
                    setColumnName={setNamespace}
                    cloudContext={{
                      cloudServiceProvider: params.cloudServiceProvider as CloudServiceProvider,
                      name: params.cloudContext ? params.cloudContext : 'ALL',
                      labelName: ''
                    }}
                  />
                </Form.Group>
                : <></>
            }
            {
              namespaceNameFlg
                ? <Form.Group style={{ marginTop: '2rem' }}>
                  <Form.Label>Namespace Name</Form.Label>
                  <FieldSelect
                    columnKey="namespace"
                    columnName={namespaceName}
                    setColumnName={setNamespaceName}
                    cloudContext={{
                      cloudServiceProvider: params.cloudServiceProvider as CloudServiceProvider,
                      name: params.cloudContext ? params.cloudContext : 'ALL',
                      labelName: ''
                    }}
                  />
                </Form.Group>
                : <></>
            }
            <Form.Group style={{ marginTop: '2rem' }}>
              <ItemCountLabel
                cloudServiceProvider={params.cloudServiceProvider}
                cloudContext={params.cloudContext}
                entityTypeId={entityTypeId}
                namespace={namespace}
                namespaceName={namespaceName}
                itemCount={itemCount}
                setItemCount={setItemCount} />
            </Form.Group>
          </Form>
        </Col>
      </Row>
    </Container>
    <PageSelector
      pageIndex={pageIndex}
      setPageIndex={setPageIndex}
      itemCount={itemCount} />
    <Container fluid className="px-0">
      <Row>
        <Col>
          <Form>
            <EntityTable
              entityTypeId={entityTypeId}
              entityColumnList={menuTemplate.entityColumn}
              namespace={namespace}
              namespaceName={namespaceName}
              pageIndex={pageIndex}
              cloudContext={params.cloudContext}
              detailInfo={
                typeof detailInfoColumn !== 'undefined'
                  ? {
                    column: detailInfoColumn,
                    path: getEntityListViewUrl(menuTemplate)
                  }
                  : undefined} />
          </Form>
        </Col>
      </Row>
    </Container>
    <PageSelector
      pageIndex={pageIndex}
      setPageIndex={setPageIndex}
      itemCount={itemCount} />
  </>;
}

export default EntityViewPage;

==> cloud-5.0.0.tar.gz: cloud/modules/cloud_dashboard/cloud_dashboard/src/pages/EntityViewPage.tsx <==
import ENTITY_INFO_LIST from 'constant/templates/entity/entity_info_template';
import { MENU_TEMPLATE_LIST } from 'constant/templates/menu/menu_template';
import CloudServiceProvider from 'model/CloudServiceProvider';
import MenuTemplate from 'model/MenuTemplate';
import ActionButtonGroup from 'molecules/ActionButtonGroup';
import EntityTabs from 'molecules/EntityTabs';
import FieldSelect from 'molecules/FieldSelect';
import ItemCountLabel from 'molecules/ItemCountLabel';
import ItemPerPageSelector from 'molecules/ItemPerPageSelector';
import PageSelector from 'molecules/PageSelector';
import Breadcrumbs from 'organisms/Breadcrumbs';
import EntityTable from 'organisms/EntityTable';
import PageTitle from 'organisms/PageTitle';
import SideBar from 'organisms/SideBar';
import { useEffect, useState } from 'react';
import { Col, Container, Form, Row } from 'react-bootstrap';
import { useParams } from 'react-router-dom';
import { getEntityListViewUrl, getEntityTypeId } from 'service/string';

const calcDetailInfoColumn = (menuTemplate: MenuTemplate) => {
  // If the entity_info_template is not registered, it will not be detected.
  const registeredTemplateList = ENTITY_INFO_LIST.filter((e) => {
    return e.cloudServiceProvider === menuTemplate.cloudServiceProvider
      && e.entityName === menuTemplate.entityName;
  });
  if (registeredTemplateList.length < 1) {
    return undefined;
  }

  // Detect a property whose name is 'name' and sets a link to it.
  const nameColumnList = menuTemplate.entityColumn.filter((e) => {
    return e.name === 'name';
  })
  if (nameColumnList.length > 0) {
    return nameColumnList[0].name;
  }

  // Detect a property whose name contains 'name' and set links to it.
  const likeNameColumnList = menuTemplate.entityColumn.filter((e) => {
    return e.name.includes('name');
  })
  if (likeNameColumnList.length > 0) {
    return likeNameColumnList[0].name;
  }

  // Default value when automatic detection is not possible.
  return undefined;
}

const EntityViewPage = () => {
  const params: {
    cloudServiceProvider: string,
    cloudContext?: string,
    entityName: string
  } = useParams();

  const [namespace, setNamespace] = useState<string>('');
  const [namespaceName, setNamespaceName] = useState<string>('');
  const [itemCount, setItemCountImpl] = useState(0);
  const [itemPerPage, setItemPerPageImpl] = useState(50);
  const [pageIndex, setPageIndex] = useState(0);

  useEffect(() => {
    if (namespace !== '') {
      setNamespaceName('');
    }
  }, [namespace]);

  useEffect(() => {
    if (namespaceName !== '') {
      setNamespace('');
    }
  }, [namespaceName]);

  useEffect(() => {
    window.scrollBy(0, -window.pageYOffset);
  }, [pageIndex]);

  const menuTemplate = MENU_TEMPLATE_LIST.filter((m) => {
    return m.cloudServiceProvider === params.cloudServiceProvider
      && m.entityName === params.entityName;
  })[0];
  const namespaceFlg = menuTemplate.entityColumn.map((c) => c.labelName).includes('Namespace');
  const namespaceNameFlg = menuTemplate.entityColumn.map((c) => c.labelName).includes('Namespace name');
  const entityTypeId = getEntityTypeId(menuTemplate);
  const detailInfoColumn = calcDetailInfoColumn(menuTemplate);

  const setItemCount = (n: number) => {
    setItemCountImpl(n);
    setPageIndex(0);
  }

  const setItemPerPage = (n: number) => {
    setItemPerPageImpl(n);
    setPageIndex(0);
  }

  return <>
    <PageTitle />
    <Breadcrumbs />
    <SideBar />
    <EntityTabs menuType={menuTemplate.cloudServiceProvider} entityName={menuTemplate.entityName} cloudContext={params.cloudContext} />
    <Container fluid className="px-0">
      <Row className="mx-0">
        <Col>
          <Form>
            {
              params.cloudContext !== undefined
                ? <ActionButtonGroup menuTemplate={menuTemplate} cloudContext={params.cloudContext} />
                : <></>
            }
            {
              namespaceFlg
                ? <Form.Group className="d-flex" style={{ marginTop: '2rem' }}>
                  <Form.Label className="text-nowrap mt-1">Namespace</Form.Label>
                  <FieldSelect
                    columnKey="namespace"
                    columnName={namespace}
                    setColumnName={setNamespace}
                    cloudContext={{
                      cloudServiceProvider: params.cloudServiceProvider as CloudServiceProvider,
                      name: params.cloudContext ? params.cloudContext : 'ALL',
                      labelName: ''
                    }}
                  />
                </Form.Group>
                : <></>
            }
            {
              namespaceNameFlg
                ? <Form.Group className="d-flex" style={{ marginTop: '2rem' }}>
                  <Form.Label className="text-nowrap mt-1">Namespace Name</Form.Label>
                  <FieldSelect
                    columnKey="namespace"
                    columnName={namespaceName}
                    setColumnName={setNamespaceName}
                    cloudContext={{
                      cloudServiceProvider: params.cloudServiceProvider as CloudServiceProvider,
                      name: params.cloudContext ? params.cloudContext : 'ALL',
                      labelName: ''
                    }}
                  />
                </Form.Group>
                : <></>
            }
            <ItemPerPageSelector
              itemPerPage={itemPerPage}
              setItemPerPage={setItemPerPage}
            />
            <Form.Group style={{ marginTop: '2rem' }}>
              <ItemCountLabel
                cloudServiceProvider={params.cloudServiceProvider}
                cloudContext={params.cloudContext}
                entityTypeId={entityTypeId}
                namespace={namespace}
                namespaceName={namespaceName}
                itemCount={itemCount}
                setItemCount={setItemCount} />
            </Form.Group>
          </Form>
        </Col>
      </Row>
    </Container>
    <PageSelector
      pageIndex={pageIndex}
      setPageIndex={setPageIndex}
      itemCount={itemCount}
      itemPerPage={itemPerPage} />
    <Container fluid className="px-0">
      <Row>
        <Col>
          <Form>
            <EntityTable
              entityTypeId={entityTypeId}
              entityColumnList={menuTemplate.entityColumn}
              namespace={namespace}
              namespaceName={namespaceName}
              pageIndex={pageIndex}
              cloudContext={params.cloudContext}
              itemPerPage={itemPerPage}
              detailInfo={
                typeof detailInfoColumn !== 'undefined'
                  ? {
                    column: detailInfoColumn,
                    path: getEntityListViewUrl(menuTemplate)
                  }
                  : undefined} />
          </Form>
        </Col>
      </Row>
    </Container>
    <PageSelector
      pageIndex={pageIndex}
      setPageIndex={setPageIndex}
      itemCount={itemCount}
      itemPerPage={itemPerPage} />
  </>;
}

export default EntityViewPage;
